package de.fhhannover.inform.trust.ifmapj;

import de.fhhannover.inform.trust.ifmapj.channel.SSRC;
import de.fhhannover.inform.trust.ifmapj.exception.IfmapErrorResult;
import de.fhhannover.inform.trust.ifmapj.exception.IfmapException;
import de.fhhannover.inform.trust.ifmapj.identifier.IdentifierFactory;
import de.fhhannover.inform.trust.ifmapj.identifier.IpAddress;
import de.fhhannover.inform.trust.ifmapj.messages.PublishRequest;
import de.fhhannover.inform.trust.ifmapj.messages.PublishUpdate;
import de.fhhannover.inform.trust.ifmapj.messages.RequestFactory;
import de.fhhannover.inform.trust.ifmapj.messages.ResultItem;
import de.fhhannover.inform.trust.ifmapj.messages.SearchRequest;
import de.fhhannover.inform.trust.ifmapj.messages.SearchResult;
import de.fhhannover.inform.trust.ifmapj.metadata.StandardIfmapMetadataFactory;

/**
 * This is _not_ a real test, but more a Hello World Example!
 * 
 * @author aw
 *
 */
public class SearchTest {

	private static RequestFactory rfac = IfmapJ.createRequestFactory();
	private static IdentifierFactory ifac = IfmapJ.createIdentifierFactory();
	private static StandardIfmapMetadataFactory mfac = IfmapJ.createStandardMetadataFactory();
	
	public static void main(String args[]) throws  IfmapErrorResult, IfmapException {
		
		PublishRequest pr1 = preparePublishRequestUpdate();
		SearchRequest sr1 = prepareSearchRequest();
		
		IfmapJHelper.setSystemProperties(
				ConfData.TRUST_STORE_PATH,
				ConfData.TRUST_STORE_PASSWORD,
				ConfData.KEY_STORE_PATH,
				ConfData.KEY_STORE_PASSWORD);
		
		SSRC ssrc = IfmapJ.createSSRC(
				ConfData.CERT_AUTH_SERVER_URL,
				IfmapJHelper.getKeyManagers(),
				IfmapJHelper.getTrustManagers());
		ssrc.newSession();
		ssrc.publish(pr1);
		//ssrc.setGzip(true);
	
		SearchResult sres = ssrc.search(sr1);
		for (ResultItem ri : sres.getResultItems()) {
			System.out.println(ri.getIdentifier1());
			System.out.println(ri.getMetadata().size());
		}
		
		ssrc.endSession();
	}

	private static PublishRequest preparePublishRequestUpdate() {
		PublishRequest ret = rfac.createPublishReq();
		PublishUpdate update = rfac.createPublishUpdate();
		IpAddress ip = ifac.createIp4("192.168.0.1");
		update.setIdentifier1(ip);
		update.addMetadata(mfac.createIpMac());
		update.addMetadata(mfac.createIpMac());
		update.addMetadata(mfac.createIpMac());
		update.addMetadata(mfac.createIpMac());
		ret.addPublishElement(update);
		return ret;
	}
	
	private static SearchRequest prepareSearchRequest() {
		SearchRequest sreq = rfac.createSearchRequest();
		IpAddress ip = ifac.createIp4("192.168.0.1");
		sreq.setStartIdentifier(ip);
		return sreq;
	}
}
