package de.fhhannover.inform.trust.ifmapj.channel;

import javax.net.ssl.SSLSocketFactory;

import de.fhhannover.inform.trust.ifmapj.exception.InitializationException;
import de.fhhannover.inform.trust.ifmapj.log.IfmapJLog;

public abstract class CommunicationHandlerFactory {
	
	/**
	 * Create a {@link CommunicationHandler} instance.
	 * 
	 * @param url The url to connect to.
	 * @param user for basic auth, if user != null -> pass != null
	 * @param pass for basic auth, if pass != null -> user != null
	 * @param sslSocketFactory the {@link SSLSocketFactory} to be used
	 * @return
	 * @throws InitializationException 
	 */
	public static CommunicationHandler newHandler(String url, String user,
			String pass, SSLSocketFactory sslSocketFactory)
			throws InitializationException {
		
		try {
			//throw new NoClassDefFoundError(" This is wanted ;) ");
			return new ApacheCoreCommunicationHandler(url, user, pass, sslSocketFactory);
		} catch (NoClassDefFoundError e) {
			IfmapJLog.warn("Could not initialize Apache Communication Handler ["
					+ e.getMessage() + "]");
		}
		IfmapJLog.warn("Using fallback to standard JDK Communication Handler");
		return new JavaCommunicationHandler(url, user, pass, sslSocketFactory);
	}
}
