package de.fhhannover.inform.trust.ifmapj.identifier;

/**
 * Factory to create different types of {@link Identifier} implementations.
 * 
 * @author aw
 */
public interface IdentifierFactory {
	
	/**
	 * Create an access-request identifier.
	 * 
	 * @return
	 */
	public AccessRequest createAr();

	
	/**
	 * Create an access-request identifier.
	 * 
	 * @param name
	 * @return
	 */
	public AccessRequest createAr(String name);

	
	/**
	 * Create an access-request identifier.
	 * 
	 * @param name
	 * @param admDom
	 * @return
	 */
	public AccessRequest createAr(String name, String admDom);

	
	/**
	 * Create a device identifier.
	 * 
	 * @return
	 */
	public Device createDev();

	
	/**
	 * Create a device identifier.
	 * 
	 * @param name
	 * @return
	 */
	public Device createDev(String name);

	
	/**
	 * Create an identity identifier.
	 * 
	 * @return
	 */
	public Identity createIdentity();

	
	/**
	 * Create an identity identifier.
	 * 
	 * @param type
	 * @param name
	 * @return
	 */
	public Identity createIdentity(IdentityType type, String name);
	
	
	/**
	 * Create an identity identifier.
	 * 
	 * @param type
	 * @param name
	 * @param admDom
	 * @return
	 */
	public Identity createIdentity(IdentityType type, String name, String admDom);

	
	/**
	 * Create an identity identifier.
	 * <br/><br/>
	 * <b>Note: otherTypeDef != null should imply type {@link IdentityType#other}.</b>
	 * 
	 * @param type
	 * @param name
	 * @param admDom
	 * @param otherTypeDef
	 * @return
	 */
	public Identity createIdentity(IdentityType type, String name, String admDom,
			String otherTypeDef);

	/**
	 * Create an identity identifier.
	 * 
	 * <b>Note: The type is set to {@link IdentityType#other} by default.</b>
	 * 
	 * @param name
	 * @param admDom
	 * @param otherTypeDef
	 * @return
	 */
	public Identity createIdentity(String name, String admDom, String otherTypeDef);

	
	/**
	 * Create an ip-address identifier for IPv4.
	 * 
	 * @return
	 */
	public IpAddress createIp4();


	/**
	 * Create an ip-address identifier for IPv4.
	 * 
	 * @param value
	 * @return
	 */
	public IpAddress createIp4(String value);
	
	/**
	 * Create an ip-address identifier for IPv4.
	 * 
	 * @param value
	 * @param admDom
	 * @return
	 */
	public IpAddress createIp4(String value, String admDom);

	/**
	 * Create an ip-address identifier for IPv6.
	 * 
	 * @return
	 */
	public IpAddress createIp6();


	/**
	 * Create an ip-address identifier for IPv6.
	 * 
	 * @param value
	 * @return
	 */
	public IpAddress createIp6(String value);
	
	/**
	 * Create an ip-address identifier for IPv6.
	 * 
	 * @param value
	 * @param admDom
	 * @return
	 */
	public IpAddress createIp6(String value, String admDom);

	
	/**
	 * Create an ip-address identifier.
	 * 
	 * @param type
	 * @param value
	 * @param admDom
	 * @return
	 */
	public IpAddress createIp(IpAddressType type, String value, String admDom);

	
	/**
	 * Create a mac-address identifier.
	 * 
	 * @return
	 */
	public MacAddress createMac();

	
	/**
	 * Create a mac-address identifier.
	 * 
	 * @param value
	 * @return
	 */
	public MacAddress createMac(String value);
	
	
	/**
	 * Create a mac-address identifier.
	 * 
	 * @param value
	 * @param admDom
	 * @return
	 */
	public MacAddress createMac(String value, String admDom);
}
