package de.fhhannover.inform.trust.ifmapj.metadata;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.fhhannover.inform.trust.ifmapj.binding.IfmapStrings;
import de.fhhannover.inform.trust.ifmapj.log.IfmapJLog;

/**
 * Simple implementation of the {@link StandardIfmapMetadataFactory} interface.
 * 
 * @author aw
 *
 */
public class StandardIfmapMetadataFactoryImpl implements StandardIfmapMetadataFactory {

	private DocumentBuilder mDocumentBuilder;
	
	public StandardIfmapMetadataFactoryImpl() {
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		dbf.setNamespaceAware(true);
		try {
			mDocumentBuilder = dbf.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			IfmapJLog.error("Could not get DocumentBuilder instance [" +
					e.getMessage() + "]");
			throw new RuntimeException(e);
		}
	}

	@Override
	public Document createIpMac(String startTime, String endTime,
			String dhcpServer) {
		Element childEl;
		Node txtEl;
		Document doc = createStdSingleElementDocument("ip-mac",
				Cardinality.multiValue);
		Element root = (Element)doc.getFirstChild();
		
		if (startTime != null) {
			childEl = doc.createElement("start-time");
			txtEl = doc.createTextNode(startTime);
			childEl.appendChild(txtEl);
			root.appendChild(childEl);
		}

		if (endTime != null) {
			childEl = doc.createElement("end-time");
			txtEl = doc.createTextNode(endTime);
			childEl.appendChild(txtEl);
			root.appendChild(childEl);
		}
		
		if (startTime != null) {
			childEl = doc.createElement("dhcp-server");
			txtEl = doc.createTextNode(dhcpServer);
			childEl.appendChild(txtEl);
			root.appendChild(childEl);
		}
		
		return doc;
	}
	
	@Override
	public Document createIpMac() {
		return createIpMac(null, null, null);
	}

	@Override
	public Document createArMac() {
		return createStdSingleElementDocument("access-request-mac",
				Cardinality.singleValue);
	}

	@Override
	public Document createArDev() {
		return createStdSingleElementDocument("access-request-device",
				Cardinality.singleValue);
	}
	
	@Override
	public Document createArIp() {
		return createStdSingleElementDocument("access-request-ip",
				Cardinality.singleValue);
	}

	@Override
	public Document createAuthAs() {
		return createStdSingleElementDocument("authenticated-as",
				Cardinality.singleValue);
	}

	@Override
	public Document createAuthBy() {
		return createStdSingleElementDocument("authenticated-by",
				Cardinality.singleValue);
	}

	@Override
	public Document createDevIp() {
		return createStdSingleElementDocument("device-ip",
				Cardinality.singleValue);
	}

	@Override
	public Document createDiscoveredBy() {
		return createStdSingleElementDocument("discovered-by",
				Cardinality.singleValue);
	}

	@Override
	public Document createRole(String name, String administrativeDomain) {
		Element childEl;
		Node txtEl;
		Document doc = createStdSingleElementDocument("role",
				Cardinality.multiValue);
		Element root = (Element)doc.getFirstChild();
		
		if (name == null)
			throw new NullPointerException("name needs to be set for role");
		
		childEl = doc.createElement("name");
		txtEl = doc.createTextNode(name);
		childEl.appendChild(txtEl);
		root.appendChild(childEl);
		
		if (administrativeDomain != null) {
			childEl = doc.createElement("administrative-domain");
			txtEl = doc.createTextNode(administrativeDomain);
			childEl.appendChild(txtEl);
			root.appendChild(childEl);
		}
		return doc;
	}
	
	@Override
	public Document createRole(String name) {
		return createRole(name, null);
	}
	
	@Override
	public Document createDevAttr(String name) {
		Element childEl;
		Node txtEl;
		Document doc = createStdSingleElementDocument("device-attribute",
				Cardinality.multiValue);
		Element root = (Element)doc.getFirstChild();
		
		if (name == null)
			throw new NullPointerException("name needs to be set for role");
		
		childEl = doc.createElement("name");
		txtEl = doc.createTextNode(name);
		childEl.appendChild(txtEl);
		root.appendChild(childEl);
		
		return doc;
	}
	
	private Document createStdSingleElementDocument(String name, Cardinality card) {
		return createSingleElementDocument(
				IfmapStrings.STD_METADATA_PREFIX + ":" + name,
				IfmapStrings.STD_METADATA_NS_URI,
				card);
	}
	
	private Document createSingleElementDocument(String qualifiedName,
			String uri, Cardinality cardinality) {
		Document doc = mDocumentBuilder.newDocument();
		Element e = doc.createElementNS(uri, qualifiedName);
		e.setAttribute("ifmap-cardinality", cardinality.toString());
		doc.appendChild(e);
		return doc;
	}
}
